# coding: UTF-8

require 'turnip/extra_steps/support/mail_finder'

::RSpec.configure do |config|
  # Waits 2 seconds after each step
  config.before do
    ActionMailer::Base.deliveries.clear
  end
end

#When /^I clear my e?mails$/ do
step "I clear my emails" do
  ActionMailer::Base.deliveries.clear
end

# Example:
#
#       Then an email should have been sent with:
#         """
#         From: max.mustermann@example.com
#         Reply-To: mmuster@gmail.com
#         To: john.doe@example.com
#         Subject: The subject may contain "quotes"
#         Attachments: ...
#
#         Message body goes here.
#         """
#
# You can skip lines, of course. Note that the mail body is only checked for
# _inclusion_.
#Then /^(an|no) e?mail should have been sent with:$/ do |mode, raw_data|

step "true with:" do
end

step "an email :whether_to have been sent with:" do |positive, raw_data|
  patiently do
    raw_data.strip!
    header, body = raw_data.split(/\n\n/, 2) # 2: maximum number of fields
    conditions = {}
    header.split("\n").each do |row|
      if row.match(/^[a-z\-]+: /i)
        key, value = row.split(": ", 2)
        conditions[key.underscore.to_sym] = value
      end
    end
    conditions[:body] = body if body
    @mail = MailFinder.find(conditions)
    if positive
      expect(@mail).to be_present
    else
      expect(@mail).not_to be_present
    end
  end
end

# nodoc
#TODO: improve regexp to this condition
#Then /^(an|no) e?mail should have been sent((?: |and|with|from "[^"]+"|bcc "[^"]+"|to "[^"]+"|the subject "[^"]+"|the body "[^"]+"|the attachments "[^"]+")+)$/ do |mode, query|
  #patiently do
    #conditions = {}
    #conditions[:to] = $1 if query =~ /to "([^"]+)"/
    #conditions[:cc] = $1 if query =~ / cc "([^"]+)"/
    #conditions[:bcc] = $1 if query =~ /bcc "([^"]+)"/
    #conditions[:from] = $1 if query =~ /from "([^"]+)"/
    #conditions[:subject] = $1 if query =~ /the subject "([^"]+)"/
    #conditions[:body] = $1 if query =~ /the body "([^"]+)"/
    #conditions[:attachments] = $1 if query =~ /the attachments "([^"]+)"/
    #@mail = MailFinder.find(conditions)
    #expectation = mode == 'no' ? 'should_not' : 'should'
    #@mail.send(expectation, be_present)
  #end
#end


step "an email :whether_to have been sent with :mode" do |positive, query|
  patiently do
    conditions = {}
    conditions[:to] = $1 if query =~ /to "([^"]+)"/
    conditions[:cc] = $1 if query =~ / cc "([^"]+)"/
    conditions[:bcc] = $1 if query =~ /bcc "([^"]+)"/
    conditions[:from] = $1 if query =~ /from "([^"]+)"/
    conditions[:subject] = $1 if query =~ /the subject "([^"]+)"/
    conditions[:body] = $1 if query =~ /the body "([^"]+)"/
    conditions[:attachments] = $1 if query =~ /the attachments "([^"]+)"/
    @mail = MailFinder.find(conditions)
    expectation = positive ? 'should' : 'should_not'
    @mail.send(expectation, be_present)
  end
end


# Only works after you have retrieved the mail using "Then an email should have been sent with:"
#When /^I follow the (first|second|third)? ?link in the e?mail$/ do |index_in_words|
step "I follow the :index_in_words link in the email" do |index_in_words|
  mail = @mail || ActionMailer::Base.deliveries.last
  index = { nil => 0, 'first' => 0, 'second' => 1, 'third' => 2 }[index_in_words]
  url_pattern = %r{(?:http|https)://[^/]+([^"'\s\\]*)}
  mail_body = MailFinder.email_text_body(mail).to_s
  only_path = mail_body.scan(url_pattern)[index][0]
  visit only_path
end

#Then /^no e?mail should have been sent$/ do
step "no email should have been sent" do
  expect(ActionMailer::Base.deliveries).to be_empty
end

# Checks that the last sent email includes some text
#Then /^I should see "([^\"]*)" in the e?mail$/ do |text|
step "I should see :text in the email" do |text|
  expect(MailFinder.email_text_body(ActionMailer::Base.deliveries.last)).to include(text)
end

# Print all sent emails to STDOUT.
#Then /^show me the e?mails$/ do
step "show me the email" do
  ActionMailer::Base.deliveries.each do |mail|
    p [mail.from, mail.to, mail.subject]
  end
end


# Only works after you've retrieved the email using "Then an email should have been sent with:"
#
# Example:
#
#       And that mail should have the following lines in the body:
#         """
#         All of these lines
#         need to be present
#         """
#Then /^that e?mail should( not)? have the following lines in the body:$/ do |negate, body|
step "that email :whether_to have the following lines in the body:" do |positive, body|
  expectation = positive ? :should : :should_not
  email_text_body = MailFinder.email_text_body(@mail)

  body.to_s.strip.split(/\n/).each do |line|
    email_text_body.send(expectation, include(line.strip))
  end
end

# Only works after you've retrieved the email using "Then an email should have been sent with:"
# Checks that the text should be included in the retrieved email
step "that email should have the following body:" do |body|
  MailFinder.email_text_body(@mail).should include(body.strip)
end
